/*
 Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 
 Author:  Sebastian Scholz <sebastian.scholz@aizo.com>
 
 This file is part of Google-Meter digitalSTROM Server add on.
 
 Google-Meter is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 Google-Meter is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

//= require <gettexthelper>
Ext.namespace('GoogleMeter', 'GoogleMeter.config');

GoogleMeter.config.VERSION = '0.1.0';

GoogleMeter.config.REGISTER_BASE_URL ="http://www.google.com/powermeter/device/activate?mfg=digitalSTROM";
GoogleMeter.config.METER_URL = "http://www.google.com/powermeter/settings";
GoogleMeter.config.GOOGLE_URL = "https://www.google.com/accounts/NewAccount";

var params = 
{
    "domain": "google-meter",
    "locale_data": json_locale_data
};

GoogleMeter.config.gt = new Gettext(params);

function _(msgid)
{
    if (GoogleMeter.config.gt === undefined) 
    {
        return msgid;
    }
    
    return GoogleMeter.config.gt.gettext(msgid);
}


GoogleMeter.MainPanel = Ext.extend(Ext.Panel, 
{
    initComponent: function()
    {
        this.configpanel = new GoogleMeter.ConfigPanel();
        
        Ext.apply(this, 
        {
            layout: 'anchor',
			hideBorders: true,
            title: _("Configure Google Meter settings"),
            items: [
			{
				autoHeight: true,
				items: new GoogleMeter.HelpPanel(),
				padding: '20'
			},
			{
				items: this.configpanel,
				padding: '20'
			}],
            bbar:
            {
                xtype: 'toolbar',
                items: [
                {
                    xtype: 'tbtext',
                    cls: 'x-black-text x-text-align-left x-text-cut',
                    text: _("Version") + ': ' + GoogleMeter.config.VERSION
                }, 
                {
                    xtype: 'tbseparator',
                    width: 8
                }, '->', 
                {
                    xtype: 'button',
                    text: _("Save configuration"),
                    ref: 'saveButton',
                    iconCls: 'icon-save',
                    scope: this,
                    handler: function()
                    {
                        var maskElement = this.findParentByType('viewport').el;
                        maskElement.mask(_("Saving") + "...", 'x-mask-loading');
                        Ext.Ajax.request(
                        {
                            url: '/json/event/raise',
                            scope: this,
                            params: 
                            {
                                name: 'google-meter.updateRegistration',
                                parameter: 'params=' + this.configpanel.getConfigString()
                            },
                            method: 'GET',
                            scope: this,
                            success: function(action)
                            {
                                try 
                                {
                                    var jsonData = Ext.util.JSON.decode(action.responseText);
                                    if (jsonData.ok) 
                                    {
                                        maskElement.unmask();
                                    }
                                    else 
                                    {
                                        Ext.Msg.alert(_("Error"), _("Failed to save configuration"));
                                    }
                                } 
                                catch (error) 
                                {
                                    Ext.Msg.alert(_("Error"), _("Failed to save configuration"));
                                }
                            },
                            failure: function(action)
                            {
                                Ext.Msg.alert(_("Error"), _("Failed to save configuration, please check if dSS is still running"));
                            }
                            
                        });
                    }
                    
                }]
            }
        });
        
        
        Ext.Ajax.request(
        {
            url: '/json/property/query?query=/apartment/dSMeters/*(dSID,name)',
            scope: this,
            disableCaching: true,
            timeout: 300000,
            method: 'GET',
            scope: this,
            success: function(action)
            {
                try 
                {
                    var jsonData = Ext.util.JSON.decode(action.responseText);
                    if (jsonData.ok) 
                    {
                        var meters = jsonData.result.dSMeters;
                        this.DSMPanel = new GoogleMeter.DSMPanel(Math.ceil(Math.sqrt(meters.length)));
                        for (var i = 0; i < meters.length; i++) 
                        {
                            this.DSMPanel.addItem(meters[i].dSID, meters[i].name);
                        }
                        this.add(
						{
							items: this.DSMPanel,
							padding: '20'
						});
                        this.doLayout();
						this.configpanel.loadConfig();
                        
                    }
                    else 
                    {
                        Ext.Msg.alert(_("Error"), _("Could not load the existing dSMs"));
                    }
                } 
                catch (error) 
                {
                    Ext.Msg.alert(_("Error"), _("Could not load the existing dSMs"));
                }
            },
            failure: function(action)
            {
                Ext.Msg.alert(_("Error"), _("Could not load the existing dSMs"));
            }
            
        });
        
        GoogleMeter.MainPanel.superclass.initComponent.apply(this, arguments);
    }
    
});

Ext.reg('googlemetermainpanel', GoogleMeter.MainPanel);


GoogleMeter.ConfigPanel = Ext.extend(Ext.form.FormPanel, 
{
    initComponent: function()
    {    
        this.apartmentNameField = new Ext.form.TextField(
        {
            fieldLabel: _("Apartment name"),
            emptyText: _("set the name of your apartment"),
            text: "",
            width: 400,
            enableKeyEvents: true
        });
        
        this.apartmentNameField.on('change', function(Field, newVal, oldVal)
        {
            this.setHref();
            if (newVal == "") 
                this.registerButton.disable();
            else 
                this.registerButton.enable();
            
            
        }, this);
        
        this.apartmentNameField.on('keypress', function(Field, eventObject)
        {
            if (eventObject.getKey() == 13) 
            {
                this.setHref();
            }
            if (this.apartmentNameField.getValue() == "") 
                this.registerButton.disable();
            else 
                this.registerButton.enable();
        }, this);
        
        this.registerButton = new GoogleMeter.LinkButton(
        {
            text: _("Register device"),
            fieldLabel: _("Register device"),
            href: GoogleMeter.config.REGISTER_BASE_URL + "&model=digitalSTROM&did=uniqueid&cvars=1&dvars=0",
            target: "_blank",
            disabled: true
        });
        
        this.registerStringArea = new Ext.form.TextArea(
        {
            fieldLabel: _("Registration string"),
            emptyText: _("paste your generated registration string here"),
            width: 400,
            grow: true
        });
        
        Ext.apply(this, 
        {
            layout: 'form',
			border: false,
            labelWidth: 250,
            padding: 10,
            items: [this.apartmentNameField, this.registerButton, this.registerStringArea]
        });
        
        GoogleMeter.ConfigPanel.superclass.initComponent.apply(this, arguments);
    },
	
	setHref: function()
	{
		var names = this.findParentByType('googlemetermainpanel').DSMPanel.getSelectedNames();
		var numberOfVariables = names.length + 1;
		var variableNames = '&title_0=' + this.apartmentNameField.getValue();

		
		for(var i =0; i<names.length; i++)
		{
            var number = i+1
			variableNames += '&title_' + number + '=' + names[i];
		}
		this.registerButton.setHref(GoogleMeter.config.REGISTER_BASE_URL + "&model=" + this.apartmentNameField.getValue().replace(/ /g, "") + "&did=uniqueid&cvars=" + numberOfVariables + "&dvars=0" + variableNames);

	},
    
    getConfigString: function()
    {
        var split = this.registerStringArea.getValue().split("&");
        var hash = split[0].split("=")[1];
        var token = split[1].split("=")[1];
        var path = split[2].split("=")[1];
        
        var params = 
        {
            'apartmentName': this.apartmentNameField.getValue(),
            'hash': hash,
            'token': token,
            'path': path,
            'dsids': Ext.util.JSON.encode(this.findParentByType('googlemetermainpanel').DSMPanel.getSelectedDSIDs())
        }
        
        return Ext.util.JSON.encode(params);
    },
    
    loadConfig: function()
    {
        Ext.Ajax.request(
        {
            url: '/json/property/getString?path=/scripts/google-meter/params',
            scope: this,
            method: 'GET',
            scope: this,
            success: function(action)
            {
                try 
                {
                    var jsonData = Ext.util.JSON.decode(action.responseText);
                    if (jsonData.ok) 
                    {
                        var params = Ext.util.JSON.decode(jsonData.result.value);
                        this.apartmentNameField.setValue(params.apartmentName);
                        var configString = 'hash=' + params.hash + '&token=' + params.token + '&path=' + params.path;
                        this.registerStringArea.setValue(configString);
                        this.findParentByType('googlemetermainpanel').DSMPanel.setChecked(Ext.util.JSON.decode(params.dsids));
                    }
                    else 
                    {
                        Ext.Msg.alert(_("Error"), _("Failed to load configuration"));
                    }
                } 
                catch (error) 
                {
                    Ext.Msg.alert(_("Error"), _("Failed to load configuration"));
                }
            },
            failure: function(action)
            {
                Ext.Msg.alert(_("Error"), _("Failed to load configuration, please check if dSS is still running"));
            }
            
        });
    }
    
});

Ext.reg('googlemeterconfigpanel', GoogleMeter.ConfigPanel);


GoogleMeter.DSMPanel = Ext.extend(Ext.form.FormPanel, 
{
    constructor: function(rows)
    {
        Ext.apply(this, 
        {
            layout: 'table',
			border: false,
            layoutConfig: 
            {
                columns: rows
            }
        });
        GoogleMeter.DSMPanel.superclass.constructor.apply(this, arguments);
    },
    
    addItem: function(dsid, name)
    {
        var Checkbox = new Ext.form.Checkbox(
        {
            boxLabel: name + ' - ' + dsid,
            hideLabel: false,
			name: dsid,
			realName: name
			
        });
        Checkbox.on('check', function(checkbox, checked)
		{
			this.findParentByType('googlemetermainpanel').configpanel.setHref();
        });
		this.add(Checkbox);
    },
    
    getSelectedDSIDs: function()
    {
        var counter = 0;
        var dsids = [];
        var checkbox;
        do 
        {
            checkbox = this.get(counter);
            counter++;
            
            if (checkbox === undefined) 
            {
                break;
            }
            
            if (checkbox.getXType() != 'checkbox') 
            {
                continue;
            }
            if (checkbox.getValue()) 
                dsids.push(checkbox.getName());
        }
        while (checkbox != undefined)
        
        return dsids;
    },
	
	getSelectedNames: function()
	{
		var counter = 0;
        var names = [];
        var checkbox;
        do 
        {
            checkbox = this.get(counter);
            counter++;
            
            if (checkbox === undefined) 
            {
                break;
            }
            
            if (checkbox.getXType() != 'checkbox') 
            {
                continue;
            }
            if (checkbox.getValue()) 
                names.push(checkbox.realName);
        }
        while (checkbox != undefined)
        
        return names;
	},
    
    setChecked: function(dsids)
    {
        var counter = 0;
        var checkbox;
        do 
        {
            checkbox = this.get(counter);
            counter++;
            
            if (checkbox === undefined) 
            {
                break;
            }
            
            if (checkbox.getXType() != 'checkbox') 
            {
                continue;
            }
            for (var i = 0; i < dsids.length; i++) 
            {
                if (checkbox.getName() == dsids[i]) 
                    checkbox.setValue(true);
            }
        }
        while (checkbox != undefined)
    }
    
});

Ext.reg('googlemeterdsmpanel', GoogleMeter.DSMPanel);


GoogleMeter.HelpPanel = Ext.extend(Ext.Panel, 
{
    initComponent: function()
    {
        
		var tab = '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;'
		this.helpText = _("This App helps you setting up your Google Meter configuration. This process is really easy if you follow the following steps:") +'<br>' +
		                _("1.") + tab + _("You need a Google account. If you do not have one already create one") + ' ' + '<a href="'+ GoogleMeter.config.GOOGLE_URL +'" target="_blank">'+ _("here")+'</a>' +'.'+ '<br>' +
						_("2.") + tab + _("Select the dSMs you want to appear in Google Meter. Your apartment will always appear in Google Meter.") + '<br>' +
						_("3.") + tab + _("Enter a name for your apartment (do not use blanks in the name!)") + '<br>' + 
						_("4.") + tab + _("Click on the register device button. A new page will open. Follow the instructions of this page until you see a long registration string.") + '<br>' +
						_("5.") + tab + _("Copy the registration string in the Textbox in the App.") + '<br>' +
						_("6.") + tab + _("Click on the \"Save configuration\" button on lower right corner of the App. Your dSS will start sending metering data to Google.") +'<br>' +
						_("7.") + tab + _("Visit your person Google Meter page") + ' ' + '<b><a href="'+ GoogleMeter.config.METER_URL +'" target="_blank">'+ _("here")+'</a></b>' + '.'; 
		
		this.helpArea = new Ext.form.Label(
		{
			html: this.helpText,
			height: '150'
		});
		
		
		Ext.apply(this, 
        {
            layout: 'fit',
			items: this.helpArea,
			border: false,
			autoHeight: true
        });
        GoogleMeter.HelpPanel.superclass.initComponent.apply(this, arguments);
    }    
});

Ext.reg('googlemeterhelppanel', GoogleMeter.HelpPanel);


Ext.onReady(function()
{
    Ext.get('start').remove();
    
    Ext.Ajax.request(
    {
        url: '/json/system/login',
        params: 
        {
            user: 'dssadmin',
            password: 'dssadmin'
        },
        scope: this,
        method: 'GET',
        disableCaching: true,
        success: function(response, options)
        {
            try 
            {
                if (response.responseText.length == 0) 
                {
                    Ext.MessageBox.alert(_("Error"), _("dSS not running"));
                }
                var jsonData = Ext.util.JSON.decode(response.responseText);
                if (jsonData.ok) 
                {
                    this.viewport = new Ext.Viewport(
                    {
                        hideBorders: true,
                        layout: 'border',
                        items: [
                        {
                            xtype: 'box',
                            region: 'north',
                            height: 80,
                            autoEl: 
                            {
                                tag: 'div',
                                html: '<a href="http://digitalstrom.org/" alt="' +
                                _("visit digitalSTROM.org") +
                                '"><img id="logo" src="images/ds_logo.gif"' +
                                'alt="dS-Logo"></a>' +
                                '<h1><a href="http://digitalstrom.org/"' +
                                ' alt="visit digitalSTROM.org">' +
                                _("digitalSTROM Server - Google-Meter") +
                                '</a></h1>'
                            }
                        }, 
                        {
                            region: 'center',
                            xtype: 'googlemetermainpanel'
                        }]
                    });
					
					Ext.MessageBox.alert(_("Warning"), _("The reported metering values are wrong. This leads to spiky graphs with even negative values. The issue is known and will be solved in a future release"));
                }
                else 
                    Ext.MessageBox.alert(_('Error'), _('Could not log in') + ': ' + jsonData.message);
            } 
            catch (err) 
            {
                Ext.MessageBox.alert(_('Error'), _('Could not log in') + ': ' + err);
            }
        },
        failure: function(response, options)
        {
            Ext.MessageBox.alert(_('Error'), _('Could not send login command to dSS'));
        }
        
    });
});



GoogleMeter.LinkButton = Ext.extend(Ext.Button, {
    template: new Ext.Template(
        '<table cellspacing="0" class="x-btn {3}"><tbody class="{4}">',
        '<tr><td class="x-btn-tl"><i>&#160;</i></td><td class="x-btn-tc"></td><td class="x-btn-tr"><i>&#160;</i></td></tr>',
        '<tr><td class="x-btn-ml"><i>&#160;</i></td><td class="x-btn-mc"><em class="{5}" unselectable="on"><a href="{6}" style="display:block" target="{7}" class="x-btn-text {2}">{0}</a></em></td><td class="x-btn-mr"><i>&#160;</i></td></tr>',
        '<tr><td class="x-btn-bl"><i>&#160;</i></td><td class="x-btn-bc"></td><td class="x-btn-br"><i>&#160;</i></td></tr>',
        '</tbody></table>').compile(),

    buttonSelector : 'a:first',

    /** 
     * @cfg String href
     * The URL to create a link for.
     */
    /** 
     * @cfg String target
     * The target for the &lt;a> element.
     */
    /** 
     * @cfg Object
     * A set of parameters which are always passed to the URL specified in the href
     */
    baseParams: {},

//  private
    params: {},

    getTemplateArgs: function() {
        return Ext.Button.prototype.getTemplateArgs.apply(this).concat(["button", this.getHref(), this.target]);
    },

    onClick : function(e){
        if(e.button != 0){
            return;
        }
        if(!this.disabled){
            if (this.fireEvent("click", this, e) == false) {
                e.stopEvent();
            } else {
                if(this.handler){
                    this.handler.call(this.scope || this, this, e);
                }
            }
        }
    },

    // private
    getHref: function() {
        var result = this.href;
        var p = Ext.urlEncode(Ext.apply(Ext.apply({}, this.baseParams), this.params));
        if (p.length) {
            result += ((this.href.indexOf('?') == -1) ? '?' : '&') + p;
        }
        return result;
    },

    /**
     * Sets the href of the link dynamically according to the params passed, and any {@link #baseParams} configured.
     * @param {Object} Parameters to use in the href URL.
     */
    setParams: function(p) {
        this.params = p;
        this.el.child(this.buttonSelector, true).href = this.getHref();
    },
	
	setHref: function(href) 
	{
		this.href = href;
		this.el.child(this.buttonSelector, true).href = this.getHref();
	}
});
Ext.reg('linkbutton', GoogleMeter.LinkButton);
